import tempfile
import urllib.parse
from logging import Logger
from requests.sessions import Session

from pyscrapers.configs import ConfigUrl, ConfigDebugUrls
from pyscrapers.core.url_set import UrlSet
from pyscrapers.core.ext_lxml import get_html_dom_content, get_element_as_bytes
from pyscrapers.workers.youtube_dl_handlers import youtube_dl_download_urls


def url_generator(url: str):
    yield url
    page = 30
    while True:
        yield f"{url}?page={page}"
        page += 30


def sxyprn_download(session: Session, logger: Logger):
    """
    This does the downloads
    :param session:
    :param logger:
    :return:
    """
    url_parsed = urllib.parse.urlparse(ConfigUrl.url)
    # noinspection PyProtectedMember
    base_url = url_parsed._replace(path="", params="", query="", fragment="").geturl()

    num_pages = None
    counter = 0
    urls = UrlSet()
    for url in url_generator(url=ConfigUrl.url):
        logger.info(f"loading [{url}]")
        response = session.get(url)
        response.raise_for_status()
        root = get_html_dom_content(response)
        if num_pages is None:
            num_pages = len(root.xpath("//div[contains(@class,'ctrl_el')]"))
            if num_pages == 0:
                num_pages = 1
        if ConfigDebugUrls.save:
            with tempfile.NamedTemporaryFile(delete=False) as f:
                logger.info(f"writing file [{f.name}]")
                f.write(get_element_as_bytes(root))
        elements = root.xpath("//a[contains(@class,'js-pop')]")
        for element in elements:
            href = element.attrib["href"]
            url = urllib.parse.urljoin(base_url, href)
            no_fluff = urllib.parse.urlparse(url)._replace(params="", query="", fragment="").geturl()
            urls.append(no_fluff)
        counter += 1
        if counter == num_pages:
            break
    session.close()
    logger.info(f"got total [{len(urls.urls_list)}] urls")
    logger.info(f"got [{urls.appended_twice}] appended twice urls")
    youtube_dl_download_urls(urls.urls_list)



import logging
import os
import urllib.parse
from collections import Counter
from typing import List

from pyscrapers.configs import ConfigDownload
from pyscrapers.core.ext_requests import download


class UrlSet:
    """ set of urls, with no duplicates. Can be downloaded """
    def __init__(self):
        """
        constructor
        """
        self.urls_set = set()
        self.urls_list = []
        self.appended_twice = 0
        self.counter_jpg = 0
        self.counter_video = 0

    def append(self, url: str) -> None:
        """
        add url to the list
        :param url:
        :return:
        """
        logger = logging.getLogger(__name__)
        logger.debug(f"collecting [{url}]")
        if url in self.urls_set:
            self.appended_twice += 1
        else:
            self.urls_set.add(url)
            self.urls_list.append(url)

    def extend(self, urls: List[str]) -> None:
        for url in urls:
            self.append(url)

    def print(self) -> None:
        """
        print the list
        :return:
        """
        for url in self.urls_list:
            print(url)

    def suggest_filename(self, suffix: str) -> str:
        if suffix == ".jpg":
            filename = f"image{self.counter_jpg:04d}.jpg"
            self.counter_jpg += 1
            return filename
        if suffix in (".mp4", ".webp"):
            filename = f"video{self.counter_video:04d}{suffix}"
            self.counter_video += 1
            return filename
        raise ValueError(f"dont know how to handle suffix [{suffix}]")

    def get_filename(self, suffix: str) -> str:
        filename = self.suggest_filename(suffix)
        while os.path.isfile(filename):
            filename = self.suggest_filename(suffix)
        return filename

    def download(self, session):
        """
        download the list
        :param session:
        :return:
        """
        skipped = 0
        downloads_per_suffix = Counter()
        logger = logging.getLogger(__name__)
        logger.info(f"got [{len(self.urls_list)}] urls")
        if ConfigDownload.download:
            for url in self.urls_list:
                parse_result = urllib.parse.urlparse(url)
                path = parse_result.path
                suffix = os.path.splitext(path)[1]
                filename = self.get_filename(suffix)
                if filename is None:
                    skipped += 1
                    logger.info(f"skipping [{path}]...")
                    continue

                logger.info(f"downloading [{path}] to [{filename}]...")
                response = session.get(url, stream=True)
                response.raise_for_status()
                downloads_per_suffix[suffix] += 1
                download(response, filename)
                logger.info(f"written [{filename}]...")
        logger.info(f"skipped [{skipped}]...")
        logger.info(downloads_per_suffix)



"""
This module is s set of utilities for this entire project
"""


import lxml
import lxml.html
import lxml.etree


def setup_prefix():
    # register regular expressions with lxml
    # this means that we can use regular expression functions like 'match'
    # by specifying 're:match' in our xpath expressions
    ns = lxml.etree.FunctionNamespace("http://exslt.org/regular-expressions")
    ns.prefix = 're'


def get_html_dom_content(response):
    """
    Get the content from a request
    :param response:
    :return:
    """
    response.raise_for_status()
    str_content = response.content.decode()
    root = lxml.html.fromstring(str_content)
    return root


def get_element_as_bytes(element):
    """
    turn xml element from etree to bytes
    :param element:
    :return:
    """
    return lxml.etree.tostring(element, pretty_print=True)


def get_element_as_string(element):
    """
    turn xml element from etree to string
    :param element:
    :return:
    """
    return lxml.etree.tostring(element, pretty_print=True).decode()


def print_element(element):
    """
    print xml elements from etree
    :param element:
    :return:
    """
    print(get_element_as_string(element))



from logging import Logger
from requests.sessions import Session


def getpocket_download(session: Session, _logger: Logger):
    """
    This does the heavy lifting
    :param session:
    :param _logger:
    :return:
    """
    headers = {
        "Origin": "https://app.getpocket.com",  # checked that this is needed
    }
    params = {
        "enable_cors": "1",  # checked that this is needed
        "consumer_key": "78809-9423d8c743a58f62b23ee85c",  # checked that this is needed
    }
    url = "https://getpocket.com/v3/get"
    response = session.post(url=url, headers=headers, params=params)
    response.raise_for_status()
    obj = response.json()
    print(obj)



from logging import Logger
from requests.sessions import Session


def netflix_download(_session: Session, _logger: Logger):
    """
    This does the downloads
    :param session:
    :param logger:
    :return:
    """
    # url = "netflix.com/xxxx"



from logging import Logger
from pyscrapers.configs import ConfigUser
from pyscrapers.core.ext_requests import ExtSession


def instagram_stories_download(_session: ExtSession, _logger: Logger):
    """
    This does the downloads
    :param session:
    :param logger:
    :return:
    """
    session = ExtSession(base="https://www.instagram.com")
    response = session.ext_get(ConfigUser.user)
    response.save_text()



def find_paths(element, json):
    paths = []
    to_search = [("", json)]
    while to_search:
        path, current = to_search.pop()
        did = False
        if isinstance(current, str):
            did = True
            if current == element:
                paths.append(path)
        if isinstance(current, dict):
            did = True
            for k, v in current.items():
                if k == element:
                    paths.append(path + f"[{k}]")
                to_search.append((path + f"[{k}]", v))
        if isinstance(current, list):
            did = True
            for i, x in enumerate(current):
                to_search.append((path + f"[{i}]", x))
        if isinstance(current, (type(None), bool, int, float)):
            did = True
        assert did, f"{current} of type {type(current)}"
    return paths



"""
Download books from audible
"""
from typing import List
from collections import OrderedDict
from logging import Logger
from bs4 import BeautifulSoup
from pyscrapers.core.ext_requests import ExtSession
from pyscrapers.configs import ConfigDebugUrls


def get_book_data(session: ExtSession, book_id: str) -> OrderedDict:
    url = f"https://www.audible.com/pb/{book_id}"
    print(f"getting {url}...")
    response = session.get_timeout(url)
    soup = BeautifulSoup(
        markup=response.res.text,
        features="lxml",
    )
    if ConfigDebugUrls.save:
        pretty = soup.prettify()
        with open("/tmp/single.html", "wt") as handle:
            handle.write(pretty)
    d = OrderedDict()
    divs = soup.find_all("h1", {"class": "bc-heading"})
    print(divs)
    assert len(divs) == 1
    d["title"] = divs[0].text
    return d


def add_book_ids(soup, book_ids) -> bool:
    divs = soup.find_all("div", {"class": "adbl-library-content-row"})
    more_titles = False
    for div in divs:
        book_id = div["id"].split("-")[-1]
        if book_id not in book_ids:
            book_ids.append(book_id)
            more_titles = True
    return more_titles


def add_links(soup, book_ids) -> bool:
    divs = soup.find_all("div", {"class": "adbl-library-content-row"})
    more_titles = False
    for div in divs:
        contents = div.contents
        assert len(contents) >= 1
        div = contents[1]
        contents = div.contents
        assert len(contents) >= 1
        div = contents[1]
        contents = div.contents
        assert len(contents) >= 1
        div = contents[1]
        contents = div.contents
        assert len(contents) >= 1
        a = contents[1]
        elem_href = a["href"]
        if elem_href not in book_ids:
            print(f"appending {elem_href}...")
            book_ids.append(elem_href)
            more_titles = True
    print(len(book_ids))
    return more_titles


def audible(_logger: Logger):
    """
    This does the downloads
    :param session:
    :param logger:
    :return:
    """
    session = ExtSession(base="https://www.audible.com")
    page = 1
    book_ids: List[str] = []
    while True:
        url = f"https://www.audible.com/library/titles?page={page}"
        print(f"getting {url}...")
        response = session.get_timeout(url)
        soup = BeautifulSoup(
            markup=response.res.text,
            features="lxml",
        )
        if ConfigDebugUrls.save:
            pretty = soup.prettify()
            with open(f"/tmp/page{page}.html", "wt") as handle:
                handle.write(pretty)

        # collect all books ids from the page
        # if not add_book_ids(soup, book_ids):
        if not add_links(soup, book_ids):
            break
        page += 1
    print(book_ids)
    # print(len(book_ids))
    # books = []
    # for book_id in book_ids:
    #     books.append(get_book_data(session, book_id))



"""
There are helpers for cookies
"""


def print_cookies(cookies, domain):
    """
    Print the cookies from a specific domain
    :param cookies:
    :param domain:
    :return:
    """
    print(domain)
    for cookie in cookies:
        print(cookie)



import os
import shutil
import urllib

import http.client
import logging

import requests
from tqdm import tqdm

from fake_useragent import UserAgent

from pyscrapers.configs import ConfigRequests
from pyscrapers.utils import get_cookies

import pyscrapers.core.ffprobe


FAIL = True


class ExtResponse:

    def __init__(self, res: requests.Response):
        self.res = res

    def raise_for_status(self):
        return self.res.raise_for_status()

    def save_text(self, filename: str = "/tmp/temp"):
        try:
            with open(filename, "wt") as handle:
                handle.write(self.res.content.decode())
        except IOError:
            os.unlink(filename)

    def save_binary(self, filename: str = "/tmp/temp") -> None:
        try:
            with open(filename, 'wb') as handle:
                self.res.raw.decode_content = True
                shutil.copyfileobj(self.res.raw, handle)
        except IOError:
            os.unlink(filename)


class ExtSession(requests.Session):
    """
    Inherit from requests.Session and add capabilities
    """
    def __init__(self, base: str = ""):
        super().__init__()
        cookies = get_cookies()
        if cookies is not None:
            self.cookies = cookies
        self.base = base
        ua = UserAgent()
        self.headers["User-Agent"] = ua.chrome

    def get_timeout(self, url: str):
        return ExtResponse(self.get(url, timeout=(ConfigRequests.connect_timeout, ConfigRequests.read_timeout)))

    def ext_get(self, url: str, *args, **kwargs):
        abs_url = urllib.parse.urljoin(self.base, url)
        ret = super().get(abs_url, *args, **kwargs)
        ret.raise_for_status()
        return ExtResponse(ret)

    def download_url(self, source: str, target: str) -> None:
        """
        Download a single url to a file
        """
        logger = logging.getLogger(__name__)
        logger.info('downloading [%s] to [%s]', source, target)
        if os.path.isfile(target):
            logger.info('skipping [%s]', target)
            return
        response = self.ext_get(source, stream=True)
        response.save_text(target)
        logger.info('written [%s]...', target)

    def download_video_if_wider(self, source: str, target: str, width: int) -> bool:
        """
        Download a video if it is wider than a certain width
        :param source:
        :param target:
        :param width:
        :param self:
        :return:
        """
        logger = logging.getLogger(__name__)
        logger.info('downloading [%s] to [%s]', source, target)
        if os.path.isfile(target):
            file_width = pyscrapers.core.ffprobe.height(target)
            if file_width >= width:
                logger.info('skipping because video with width exists [%s] %s %s', target, file_width, width)
                return True
            logger.info('continuing with download because of width [%s] %s %s', target, file_width, width)
        response = self.ext_get(source, stream=True)
        response.save_binary(target)
        logger.info('written [%s]...', target)
        return True


def setup():
    """
    Activate the debugging features of the requests module
    :return:
    """
    if ConfigRequests.debug:
        http.client.HTTPConnection.debuglevel = 1
        requests_log = logging.getLogger("requests.packages.urllib3")
        requests_log.propagate = True


CONTENT_LENGTH_HEADER = 'content-length'
BLOCK_SIZE = 1024 * 1024


def download(response, filename: str) -> None:
    if CONTENT_LENGTH_HEADER in response.headers:
        total_size_in_bytes = int(response.headers.get('content-length', 0))
        assert total_size_in_bytes != 0
        progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True, disable=not ConfigRequests.progress)
        have_total = True
    else:
        progress_bar = tqdm(unit='iB', unit_scale=True, disable=not ConfigRequests.progress)
        have_total = False
    # pylint: disable=broad-except
    try:
        with open(filename, "wb") as file_handle:
            for data in response.iter_content(BLOCK_SIZE):
                progress_bar.update(len(data))
                file_handle.write(data)
    except (Exception, KeyboardInterrupt, SystemError) as e:
        os.unlink(filename)
        raise e
    progress_bar.close()
    if have_total and ConfigRequests.progress:
        assert progress_bar.n == total_size_in_bytes, f"something wrong {progress_bar.n} =! {total_size_in_bytes}"



"""
This module is s set of utilities for this entire project
"""


import urllib.parse
import http.client


def get_http_status_string(code: int):
    """
    This function returns a description of an HTTP status code (404 - not found etc).
    Unfortunately, the requests module does not provide a clean API for this so
    we must access a protected member (underscore member) of 'requests.status_code'.
    See:
    https://stackoverflow.com/questions/24718557/get-the-description-of-a-status-code-in-python-requests
    :param code:
    :return:
    """
    # noinspection PyProtectedMember,PyUnresolvedReferences
    # pylint: disable=protected-access
    # return f"http code [{code}], [{requests.status_codes._codes[code][0]}]"
    return f"http code [{code}], [{http.client.responses[code]}]"


def add_http(url, main_url):
    """
    add two urls together
    :param url:
    :param main_url:
    :return:
    """
    return urllib.parse.urljoin(main_url, url)



import setuptools


def get_readme():
    with open("README.rst") as f:
        return f.read()


setuptools.setup(
    # the first three fields are a must according to the documentation
    name="pyscrapers",
    version="0.0.67",
    packages=[
        "pyscrapers",
        "pyscrapers.core",
        "pyscrapers.workers",
    ],
    # from here all is optional
    description="project to produce various useful scrapers",
    long_description=get_readme(),
    long_description_content_type="text/x-rst",
    author="Mark Veltzer",
    author_email="mark.veltzer@gmail.com",
    maintainer="Mark Veltzer",
    maintainer_email="mark.veltzer@gmail.com",
    keywords=[
        "scrape",
        "images",
        "social",
        "facebook",
        "instagram",
        "vk",
        "download",
        "pics",
    ],
    url="https://veltzer.github.io/pyscrapers",
    download_url="https://github.com/veltzer/pyscrapers",
    license="MIT",
    platforms=[
        "python3",
    ],
    install_requires=[
        "lxml",
        "requests",
        "browser-cookie3",
        "dbus-python",
        "pytconf",
        "pylogconf",
        "pornhub-api",
        "youtube-dl",
        "pyeventroute",
        "fake-useragent",
        "tqdm",
        "beautifulsoup4",
        "pydantic==1.10.13",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.11",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"console_scripts": [
        "pyscrapers=pyscrapers.main:main",
    ]},
)



import browser_cookie3


from pyscrapers.configs import ConfigCookiesSource


def get_cookies():
    if ConfigCookiesSource.browser == "firefox":
        return browser_cookie3.firefox()
    if ConfigCookiesSource.browser == "chrome":
        return browser_cookie3.chrome()
    return None
    # raise ValueError(f"unsupported browser [{ConfigCookiesSource.browser}")



import json

from pyscrapers.core.url_set import UrlSet


def scrape_mambaru(user_id: str, session, url_set: UrlSet) -> None:
    # raise ValueError("mamba still not implemented")
    # main_url = 'https://www.mamba.ru/{user_id}'.format(user_id=user_id)
    main_url = 'https://www.mamba.ru/mobile/api/v5.17.0.0/?reqType=json'
    sys_requests_container = [
        {
            "uri": f"/users/{user_id}/albums/workers/",
            "method": "GET",
            "params": {
                "langId": "en",
                "dateType": "timestamp",
                "limit": -1,
            },
        }
    ]
    request_obj = {
        "langId": "en",
        "dateType": "timestamp",
        "limit": 10000,
        "sysRequestsContainer": sys_requests_container,
    }
    response = session.post(main_url, json=request_obj)
    response.raise_for_status()
    response_str = response.content.decode()
    response_obj = json.loads(response_str)
    response_obj = response_obj["sysResponsesContainer"][0]
    for album in response_obj["albums"]:
        for photo in album["workers"]:
            url_set.append(photo["hugePhotoUrl"])
